︠4d81719d-05b2-4f68-99c4-87db34507091is︠
%hide
%html

<H3  ALIGN="CENTER"><B><FONT COLOR="#030856">
Curve fitting, including linear regression
</FONT></B></H3>


<P>
In this module we look at how to find the curve of best-fit for 
a given set of $(x,y)$ data values. From a modeling point of 
view this is a critically important need, with a wide variety 
of applications.  Linear regression is a particularly 
important "special case" of curve fitting, in which we seek 
the best straight line to fit our data.
<P><BR>

In all the examples shown here, we will use a specific dataset 
from an input 
file. In this dataset the $x$-variable is the calendar year, 
and the $y$-variable is the global mean temperature on the 
earth's surface.  The following code segment reads the data 
and stores it in the variables: <TT>xdata</TT>, <TT>ydata</TT>. <BR>
After that it plots the data to see what it looks like.
︡3eb44519-d3e1-4dcd-81bf-bc362894ec1e︡{"hide":"input"}︡{"html":"\n<H3  ALIGN=\"CENTER\"><B><FONT COLOR=\"#030856\">\nCurve fitting, including linear regression\n</FONT></B></H3>\n\n\n<P>\nIn this module we look at how to find the curve of best-fit for \na given set of $(x,y)$ data values. From a modeling point of \nview this is a critically important need, with a wide variety \nof applications.  Linear regression is a particularly \nimportant \"special case\" of curve fitting, in which we seek \nthe best straight line to fit our data.\n<P><BR>\n\nIn all the examples shown here, we will use a specific dataset \nfrom an input \nfile. In this dataset the $x$-variable is the calendar year, \nand the $y$-variable is the global mean temperature on the \nearth's surface.  The following code segment reads the data \nand stores it in the variables: <TT>xdata</TT>, <TT>ydata</TT>. <BR>\nAfter that it plots the data to see what it looks like.\n"}︡{"done":true}︡
︠7febba76-7c09-42b5-a290-44cd6a9eb64ds︠
f = open('./globtemp.csv')  # This file has comma separated data, with a plain text header line.
xdata = []                  # This will assemble my x-data in the form of a list
ydata = []                  # This will assemble my y-data in the same form
headerline = f.readline()   # Read & discard the header line in the input file
line = f.readline()         # This is the first line of actual data
while (line !=''):
    xy = line.split(',')    # The comma splits each line into two items of data
    xdata.append(float(xy[0]))  # Append to xdata
    ydata.append(float(xy[1]))  # Append to ydata
    line = f.readline()         # Read the next line

# Next, plot the data and see what it looks like:
point(zip(xdata, ydata), axes_labels=['year', 'temp $^\circ$C'], pointsize=30)
︡dd9297b7-b4fd-446f-9690-e74ddcf43c87︡{"file":{"filename":"/projects/5af60d30-8ee0-4c83-b7d2-6a0b68f44907/.sage/temp/compute3-us/29944/tmp_PuS08p.svg","show":true,"text":null,"uuid":"4c55b06b-9d1c-4490-b7a4-f969a4a7f49d"},"once":false}︡{"done":true}︡
︠e979bb67-15c9-4690-9df2-0591c40813c2is︠

%hide
%html
<FONT COLOR="#000080" SIZE="+1">
<B>Curve fitting methods</B>
</FONT>
<P><BR>
There are at least 3 different options that Sage offers 
for doing linear regression.  Keep in mind that there 
is only one unique line of best-fit for any given dataset. 
This is because the line of best-fit minimizes the sum of 
the squares of the errors.  
So, we expect all the implementations to give the same answer!   
<P><BR>


<FONT COLOR="#000080" SIZE="+1">
<B>Method 1</B></FONT>: &nbsp; Using <TT>stats.linregress</TT>

<P>
As the name suggests, this method only does linear regression 
-- as opposed to fitting other (nonlinear) models to the data. 
<BR>
The code segment below shows how to use it to fit the global 
temperature dataset.  In addition, it shows how to plot the results.
︡71b02b15-e1c1-4c22-9b7a-abf7040da7aa︡{"hide":"input"}︡{"html":"<FONT COLOR=\"#000080\" SIZE=\"+1\">\n<B>Curve fitting methods</B>\n</FONT>\n<P><BR>\nThere are at least 3 different options that Sage offers \nfor doing linear regression.  Keep in mind that there \nis only one unique line of best-fit for any given dataset. \nThis is because the line of best-fit minimizes the sum of \nthe squares of the errors.  \nSo, we expect all the implementations to give the same answer!   \n<P><BR>\n\n\n<FONT COLOR=\"#000080\" SIZE=\"+1\">\n<B>Method 1</B></FONT>: &nbsp; Using <TT>stats.linregress</TT>\n\n<P>\nAs the name suggests, this method only does linear regression \n-- as opposed to fitting other (nonlinear) models to the data. \n<BR>\nThe code segment below shows how to use it to fit the global \ntemperature dataset.  In addition, it shows how to plot the results."}︡{"done":true}︡
︠2154fcc8-7bb1-4304-b78c-f894f6f0a7c5s︠
# The library "stats" must first be imported from scipy:
from scipy import stats

# Give xdata, ydata as input to stats.linregress.
# Store the results in some variable (e.g., fit_output).
fit_output = stats.linregress(xdata, ydata)
#slope, intercept, r_value, p_value, slope_std_error = fit_output
print fit_output

# The rest of this code just plots and shows the results 
from scipy import polyval
ymod = polyval([fit_output[0],fit_output[1]],xdata)  # Compute y-values for the linear model
praw = point(zip(xdata, ydata), pointsize=30)        # Plot the original dataset
                                                     # Plot the fitted model
pfit = points(zip(xdata,ymod), axes_labels=['year', 'temp $^\circ$C'], pointsize=8, color='darkgreen')
show(praw+pfit)
print "The mathematical form of the model is: y =", fit_output[0], "* x + ", fit_output[1]
︡54a39ba7-4182-4edc-b1c8-0970eb4a5c83︡{"stdout":"LinregressResult(slope=0.0050534351145038172, intercept=5.1333435114503807, rvalue=0.81149837783481804, pvalue=6.8562147164742632e-32, stderr=0.00032039129165884194)\n"}︡{"file":{"filename":"/projects/5af60d30-8ee0-4c83-b7d2-6a0b68f44907/.sage/temp/compute3-us/18085/tmp_hlS6jy.svg","show":true,"text":null,"uuid":"ce198769-96d2-465b-b81f-ec2bc16e99bd"},"once":false}︡{"stdout":"The mathematical form of the model is: y = 0.0050534351145 * x +  5.13334351145\n"}︡{"done":true}︡
︠297f9640-4e78-4694-82b0-81cf27b07fabsi︠

%hide
%html

<FONT COLOR="#000080" SIZE="+1">
<B>Method 2</B></FONT>: &nbsp; Using <TT>polyfit</TT> from <TT>scipy</TT>

<P><BR>
The <TT>polyfit</TT> function can fit a polynomial of any 
specified order. Thus it can be used for linear regression, 
as well as for higher order regression. It returns the best 
least-squares fit to the dataset.
<P>
The code segment below illustrates how to use it to fit a linear, 
and a 3rd order polynomial to the global 
temperature dataset.  In addition, it shows plots of the results.
︡c2811b61-2b2b-4066-bd2d-ddaeeccf6bd7︡{"hide":"input"}︡{"html":"\n<FONT COLOR=\"#000080\" SIZE=\"+1\">\n<B>Method 2</B></FONT>: &nbsp; Using <TT>polyfit</TT> from <TT>scipy</TT>\n\n<P><BR>\nThe <TT>polyfit</TT> function can fit a polynomial of any \nspecified order. Thus it can be used for linear regression, \nas well as for higher order regression. It returns the best \nleast-squares fit to the dataset.\n<P>\nThe code segment below illustrates how to use it to fit a linear, \nand a 3rd order polynomial to the global \ntemperature dataset.  In addition, it shows plots of the results."}︡{"done":true}︡
︠4103041a-795b-4b9a-9135-7bcc67993533s︠
# The function "polyfit" must first be imported from scipy:
from scipy import polyfit

# Let's first do a linear fit:
fit_out_linear = polyfit(xdata, ydata, 1)
# Next, we'll try a cubic fit for the same data:
fit_out_cubic = polyfit(xdata, ydata, 3)


# The rest of this code just plots and shows the results.
# NOTE: polyfit seems to give output in format for direct input to polyval

from scipy import polyval
ymod_linear = polyval(fit_out_linear, xdata)         # Compute y-values for the linear model
ymod_cubic = polyval(fit_out_cubic, xdata)           # Compute y-values for cubic model
praw = point(zip(xdata, ydata), pointsize=30)        # Plot the original dataset
                                                     # Plot the fitted models
pfit_linear = points(zip(xdata,ymod_linear), pointsize=8, color='darkgreen')
pfit_cubic = points(zip(xdata,ymod_cubic), axes_labels=['year', 'temp $^\circ$C'], 
                    pointsize=8, color='darkred')
show(praw+pfit_linear+pfit_cubic)

print "The linear model is: y =", fit_out_linear[0], "* x + ", fit_out_linear[1]
print "The cubic model is: y =", fit_out_cubic[0], "* x^3 + ", fit_out_cubic[1], "* x^2 + ", fit_out_cubic[2], "* x + ", fit_out_cubic[3]

︡6cdec2c4-e65d-463c-9147-d996cdf1961d︡{"file":{"filename":"/projects/5af60d30-8ee0-4c83-b7d2-6a0b68f44907/.sage/temp/compute3-us/29944/tmp_H9p5qB.svg","show":true,"text":null,"uuid":"745740dc-5e4c-4d01-b9f0-9226fd12c41d"},"once":false}︡{"stdout":"The linear model is: y = 0.0050534351145 * x +  5.13334351145\n"}︡{"stdout":"The cubic model is: y = 4.05660084499e-07 * x^3 +  -0.00235611960143 * x^2 +  4.56551474091 * x +  -2936.55073889\n"}︡{"done":true}︡
︠2fc0f9c9-db4f-4a8e-a299-72179ba712f5is︠
%hide
%html

<FONT COLOR="#000080" SIZE="+1">
<B>Method 3</B></FONT>: &nbsp; Using <TT>curve_fit</TT> from <TT>scipy.optimize</TT>

<P><BR>
The <TT>curve_fit</TT> function let's the user define any type 
of model function, including linear, polynomial,  trigonometric, 
exponential, etc.  It returns the best least-squares fit of the 
model function to the given dataset.  This method requires a bit 
more coding, since it allows for more general models.
<P>
The code segment below illustrates how to use it to fit a linear, 
and a trigonometric function to the global temperature dataset. 
Note that the user must define a prototype function of each 
kind before <TT>curve_fit</TT> can be used.  The inputs to 
<TT>curve_fit</TT> are the name of the function prototype, 
and the $x$, $y$ data sets.
︡2be1df37-0582-4d45-800b-93f7857777b0︡{"hide":"input"}︡{"html":"\n<FONT COLOR=\"#000080\" SIZE=\"+1\">\n<B>Method 3</B></FONT>: &nbsp; Using <TT>curve_fit</TT> from <TT>scipy.optimize</TT>\n\n<P><BR>\nThe <TT>curve_fit</TT> function let's the user define any type \nof model function, including linear, polynomial,  trigonometric, \nexponential, etc.  It returns the best least-squares fit of the \nmodel function to the given dataset.  This method requires a bit \nmore coding, since it allows for more general models.\n<P>\nThe code segment below illustrates how to use it to fit a linear, \nand a trigonometric function to the global temperature dataset. \nNote that the user must define a prototype function of each \nkind before <TT>curve_fit</TT> can be used.  The inputs to \n<TT>curve_fit</TT> are the name of the function prototype, \nand the $x$, $y$ data sets.\n"}︡{"done":true}︡
︠b40a82a3-b931-44ad-9f7c-4abcc0c3b157s︠

# User-defined model of linear function:
def f_linear(x, m, b):
    return m*x + b

# User-defined model of trigonometric function:
def f_exp(x, a, b, c, d):
    return a + b*x + c*sin(d*x)

# Import the "curve_fit" function from scipy.optimize:
from scipy.optimize import curve_fit

# Find the best-fit linear model; save results in temporary variables:
coefs, pcov = curve_fit(f_linear, xdata, ydata)

# Compute y-values for the linear model:
ymodel = []
for i in range(len(xdata)):
    ymodel.append(f_linear(xdata[i], coefs[0], coefs[1]))

# Plot original data and linear fit:
praw = point(zip(xdata, ydata), pointsize=30)
pfit_linear = point(zip(xdata, ymodel), pointsize=8, color='darkgreen')

# Find the best-fit trigonometric model; save results in temporary variables:
coefs, pcov = curve_fit(f_exp, xdata, ydata)

# Compute y-values for the model:
ymodel = []
for i in range(len(xdata)):
    ymodel.append(f_exp(xdata[i], coefs[0], coefs[1], coefs[2], coefs[3]))

# Plot trigonometric fit:
pfit_exp = point(zip(xdata, ymodel), axes_labels=['year', 'temp $^\circ$C'], pointsize=8, color='darkred')

show(praw+pfit_linear+pfit_exp)

︡274cff8f-7570-48dc-9907-4c22c5a33354︡{"file":{"filename":"/projects/5af60d30-8ee0-4c83-b7d2-6a0b68f44907/.sage/temp/compute3-us/10354/tmp_caWwF3.svg","show":true,"text":null,"uuid":"792aaa1e-7b3c-4f8d-9578-e49399c4bf4f"},"once":false}︡{"done":true}︡
︠0095d779-69c5-40ec-aa0d-d2f32b9f27dfis︠
%hide
%html
<FONT COLOR="#000080" SIZE="+1">
<B>Exercise</B>
</FONT>
<P><BR>
The file <A HREF="./Thailand_tourism_2009_2016.csv">Thailand_tourism_2009_2016.csv</A> 
contains data from the Department of Tourism 
on total monthly foreign tourist arrivals in Thailand between the 
years 2009 and 2012.  The data is in the form of a comma-separated 
spreadsheet with 3 coulmns: number of tourists, year, month.  The 
units used for the number of tourists is millions, and the months 
are numbered consecutively starting from 1 to 96.  
The first line in the file 
is a header containing coulmn headings. The actual data start from 
the 2nd line.
<P>

The goal of this exercise is to read the necessary data from 
the input file and fit two different models to it: (1) linear, 
and (2) any one nonlinear model.  
<P>
Plot your results in the form of graphs, and also give each 
model's mathematical form.


︡66513dc8-093b-4551-af94-1d99305562f9︡{"hide":"input"}︡{"html":"<FONT COLOR=\"#000080\" SIZE=\"+1\">\n<B>Exercise</B>\n</FONT>\n<P><BR>\nThe file <A HREF=\"./Thailand_tourism_2009_2016.csv\">Thailand_tourism_2009_2016.csv</A> \ncontains data from the Department of Tourism \non total monthly foreign tourist arrivals in Thailand between the \nyears 2009 and 2012.  The data is in the form of a comma-separated \nspreadsheet with 3 coulmns: number of tourists, year, month.  The \nunits used for the number of tourists is millions, and the months \nare numbered consecutively starting from 1 to 96.  \nThe first line in the file \nis a header containing coulmn headings. The actual data start from \nthe 2nd line.\n<P>\n\nThe goal of this exercise is to read the necessary data from \nthe input file and fit two different models to it: (1) linear, \nand (2) any one nonlinear model.  \n<P>\nPlot your results in the form of graphs, and also give each \nmodel's mathematical form.\n\n\n"}︡{"done":true}︡
︠38fab04c-5596-4825-817d-abc9257c543e︠










