︠51c04dd8-da74-4653-847d-74aa6bd56946si︠
%hide
%html
<H4>Some experiments in linear programming (MILP)
</H4>

Example 1: <BR>
&nbsp; &nbsp; &nbsp; &nbsp; 
$\begin{array}{ll} \mbox{maximize:} & x+y+3z \\ 
      \mbox{subject to:} & x+2y \le 4  \\
             & 5z-y \le 8  \\
             & x, y, z \ge 0 \end{array}$

︡b709e56f-9d0b-4e6e-9a9c-e381bd18acc5︡{"hide":"input"}︡{"html":"<H4>Some experiments in linear programming (MILP)\n</H4>\n\nExample 1: <BR>\n&nbsp; &nbsp; &nbsp; &nbsp; \n$\\begin{array}{ll} \\mbox{maximize:} & x+y+3z \\\\ \n      \\mbox{subject to:} & x+2y \\le 4  \\\\\n             & 5z-y \\le 8  \\\\\n             & x, y, z \\ge 0 \\end{array}$"}︡{"done":true}︡
︠6eaac529-cd21-4661-beb0-53e51b50ceces︠
# Declare a MILP object, say, r.
r = MixedIntegerLinearProgram()

# Declare the variables associated with r.
# Note that we're also imposing the nonnegativity constraint simultaneously.
x = r.new_variable(real=True, nonnegative=True)

# Define the objective function.
r.set_objective(x[1]+x[2]+3*x[3])

# Impose the remaining constraints.
r.add_constraint(x[1]+2*x[2] <= 4)
r.add_constraint(5*x[3]-x[2] <= 8)

# solve and print results
a = r.solve()
print "maximum value =", a
b = r.get_values(x)
print "values of variables are =", b
︡02667acb-f98d-4442-93c5-dba180ef105e︡{"stdout":"maximum value = 8.8\n"}︡{"stdout":"values of variables are = {1: 4.0, 2: 0.0, 3: 1.6}\n"}︡{"done":true}︡
︠eeb38512-2feb-46fd-a002-7b7ed1528787is︠
%hide
%html
Example 2: <P>
The next example is an attempt to setup an optimization problem for 
the HAAI test case along lines suggested by อาจารย์พัชรี.

︡d9e2d407-1c57-4add-9d50-dee2792788c5︡{"hide":"input"}︡{"html":"Example 2: <P>\nThe next example is an attempt to setup an optimization problem for \nthe HAAI test case along lines suggested by อาจารย์พัชรี."}︡{"done":true}︡
︠a3c72cfb-1188-45be-819d-84096417da13s︠
# Define channel capacities along each edge in the form of a dictionary:
c = {(0,1):7, (1,2):4, (1,3):4, (2,4):2, (4,6):4, (3,5):4, (5,6):2.5, (5,7):2.5, (6,8):5, (7,9):3}
# Test whether it works correctly:
c[(5,7)]
# For minimization problems, set the input variable "maximization" to False
r = MixedIntegerLinearProgram(maximization=False)

# Declare the variables associated with r.  
# I'll use "p" for pumping, and "f" for flow rate
p = r.new_variable(real=True, nonnegative=True)
f = r.new_variable(real=True, nonnegative=True)

# Define the objective function.
r.set_objective(p[(0,1)]+p[(1,2)]+p[(1,3)]+p[(2,4)]+p[(4,6)]+p[(3,5)]+p[(5,6)]+p[(5,7)]+p[(6,8)]+p[(7,9)])

# Impose the node/flow constraints.
r.add_constraint( f[(0,1)] == f[(1,2)]+f[(1,3)] )
r.add_constraint( f[(1,2)] == f[(2,4)] )
r.add_constraint( f[(1,3)] == f[(3,5)] )
r.add_constraint( f[(2,4)] == f[(4,6)] )
r.add_constraint( f[(3,5)] == f[(5,6)]+f[(5,7)] )
r.add_constraint( f[(4,6)]+f[(5,6)] == f[(6,8)] )
r.add_constraint( f[(5,7)] == f[(7,9)] )

# Impose capacity constraints.
r.add_constraint( f[(0,1)] <= c[(0,1)] )
r.add_constraint( f[(1,2)] <= c[(1,2)] )
r.add_constraint( f[(1,3)] <= c[(1,3)] )
r.add_constraint( f[(2,4)] <= c[(2,4)] )
r.add_constraint( f[(4,6)] <= c[(4,6)] )
r.add_constraint( f[(3,5)] <= c[(3,5)] )
r.add_constraint( f[(5,6)] <= c[(5,6)] )
r.add_constraint( f[(5,7)] <= c[(5,7)] )
r.add_constraint( f[(6,8)] <= c[(6,8)] )
r.add_constraint( f[(7,9)] <= c[(7,9)] )

#r.add_constraint( f[(0,1)] <= min(7,4) )

# Attempt to impose IF-condition in the form of a "maximum" condition
#r.add_constraint( p[(0,1)] == max(0, f[(0,1)]-c[(0,1)]) )


# solve and print results
a = r.solve()
print "minimum value =", a
b = r.get_values(p)
c = r.get_values(f)
print "values of pump variables are =", b
print "values of flow variables are =", c
︡53b44dca-c5b3-4fb6-9cb6-8dec972632f7︡{"stdout":"2.50000000000000\n"}︡{"stdout":"minimum value = 0.0\n"}︡{"stdout":"values of pump variables are = {(0, 1): 0.0, (1, 2): 0.0, (1, 3): 0.0, (4, 6): 0.0, (5, 6): 0.0, (5, 7): 0.0, (3, 5): 0.0, (7, 9): 0.0, (2, 4): 0.0, (6, 8): 0.0}\n"}︡{"stdout":"values of flow variables are = {(0, 1): 0.0, (1, 2): 0.0, (1, 3): 0.0, (4, 6): -0.0, (5, 6): -0.0, (5, 7): 0.0, (3, 5): 0.0, (7, 9): -0.0, (2, 4): 0.0, (6, 8): -0.0}\n"}︡{"done":true}︡












