︠a383fda4-7046-4839-b8b4-213f2596360cis︠
%hide
%html

<H3  ALIGN="CENTER"><B><FONT COLOR="#030856">
A sampler of Sage mathematics
</FONT></B></H3>

Sage offers tools for learning and for doing research 
in many different areas of mathematics.  The purpose of 
this module is to provide a very brief glimpse into some 
these areas, including:  <P>
<OL>
    <LI>Multivariate calculus
    <LI>Linear algebra
    <LI>Ordinary differential equations
    <LI>Abstract algebra/ Groups
</OL>
︡21c4da01-3b32-443a-98c3-49765933e93a︡{"hide":"input"}︡{"html":"\n<H3  ALIGN=\"CENTER\"><B><FONT COLOR=\"#030856\">\nA sampler of Sage mathematics\n</FONT></B></H3>\n\nSage offers tools for learning and for doing research \nin many different areas of mathematics.  The purpose of \nthis module is to provide a very brief glimpse into some \nthese areas, including:  <P>\n<OL>\n    <LI>Multivariate calculus\n    <LI>Linear algebra\n    <LI>Ordinary differential equations\n    <LI>Abstract algebra/ Groups\n</OL>"}︡{"done":true}︡
︠33839c6a-d593-4ec1-aa23-da7823717292si︠
%hide
%html

<H4><B><FONT COLOR="#030856">
Functions of 2 or more variables
</FONT></B></H4>

There are several ways to define them.
The easiest and most intuitive way is shown in 
the example below. it defines a function, evaluates it at a point, 
and takes its partial derivatives.
<P><BR>

<B>Example 1</B>: Let $f(x,y,z)=(x^2+y^2)z$. Evaluate the function at 
some point in its domain, and find its 1st partial derivatives.
︡64a1c049-c65b-4e3c-9fba-7254ca6eecdb︡{"hide":"input"}︡{"html":"\n<H4><B><FONT COLOR=\"#030856\">\nFunctions of 2 or more variables\n</FONT></B></H4>\n\nThere are several ways to define them.\nThe easiest and most intuitive way is shown in \nthe example below. it defines a function, evaluates it at a point, \nand takes its partial derivatives.\n<P><BR>\n\n<B>Example 1</B>: Let $f(x,y,z)=(x^2+y^2)z$. Evaluate the function at \nsome point in its domain, and find its 1st partial derivatives."}︡{"done":true}︡
︠cfeabba0-54ac-4a73-af82-261547203a4fs︠
# A function of 3 variables: 
f(x,y,z) = (x^2 + y^2) * z

# Can evaluate the function at any point:
f(1,0,2)

# There are several ways to get partial derivatives:
f.diff(x)
f.derivative(x)
derivative(f,x)
derivative(f,z)
# Note that the answer is returned in function notation, 
# with inputs to the left and outputs to the right of the arrow.

# It is easy to compute the gradient of f:
f.gradient()
︡9ed4397c-3e45-4fe8-8e54-1080eb9ae74b︡{"stdout":"2\n"}︡{"stdout":"(x, y, z) |--> 2*x*z\n"}︡{"stdout":"(x, y, z) |--> 2*x*z\n"}︡{"stdout":"(x, y, z) |--> 2*x*z\n"}︡{"stdout":"(x, y, z) |--> x^2 + y^2\n"}︡{"stdout":"(x, y, z) |--> (2*x*z, 2*y*z, x^2 + y^2)"}︡{"stdout":"\n"}︡{"done":true}︡
︠c2709bb0-d0ca-4089-bc6e-b241be5c06f9is︠

%hide
%html

<H4><B><FONT COLOR="#030856">Vector functions of 2 or more variables
</FONT></B></H4>

When the dependent variable of a function is also 
a vector, we have the more general case.  Here are some examples <P><BR>
${\bf f}(x,y) = \left[\begin{array}{c}  x^2 + y^2  \\ 3 x y \end{array}\right ]$, 
&nbsp; &nbsp; &nbsp;
${\bf g}(x,y,z) = \left[\begin{array}{c} x^2 y  \\ x^2 z  \\ y z \end{array}\right ]$ 
<P><BR>

Such functions offer more interesting and fun variations 
for applying familiar calculus concepts!  For example, taking 
their derivatives can be approached in the form of a "divergence" 
or a "curl," with very different conceptual meaning and results. 
<P>

The Sage segment below defines each of these functions, 
evaluates them at a point, and computes their divergence and curl.
︡02c058ba-b2b6-47ef-ba5e-58db560832c3︡{"hide":"input"}︡{"html":"\n<H4><B><FONT COLOR=\"#030856\">Vector functions of 2 or more variables\n</FONT></B></H4>\n\nWhen the dependent variable of a function is also \na vector, we have the more general case.  Here are some examples <P><BR>\n${\\bf f}(x,y) = \\left[\\begin{array}{c}  x^2 + y^2  \\\\ 3 x y \\end{array}\\right ]$, \n&nbsp; &nbsp; &nbsp;\n${\\bf g}(x,y,z) = \\left[\\begin{array}{c} x^2 y  \\\\ x^2 z  \\\\ y z \\end{array}\\right ]$ \n<P><BR>\n\nSuch functions offer more interesting and fun variations \nfor applying familiar calculus concepts!  For example, taking \ntheir derivatives can be approached in the form of a \"divergence\" \nor a \"curl,\" with very different conceptual meaning and results. \n<P>\n\nThe Sage segment below defines each of these functions, \nevaluates them at a point, and computes their divergence and curl."}︡{"done":true}︡
︠709eafbb-d4a7-4b13-8233-b2cee1f15fb8s︠
# Here's how to define vector functions of multiple variables. 
# Notice, it is an intuitive generalization of the previous syntax.
#
x, y, z = var('x, y, z')
f = vector( (x^2 + y^2, 3*x*y))
g = vector( (x^2 * y, x^2 * z, y*z) )

# You can evaluate the functions at any specific point by 
# similarly generalizing the single variable syntax.
# (I've used the print command just to show the output clearly):
print "f(0,2)=",  f(x=0, y=2)
print "g(1,2,3)=", g(x=1, y=2, z=3)

# Find divergence and curl. Note, the syntax requires 
# specifying the names of the variables in order.  (Again, 
# the print command is just to show the output clearly):
print "div(f)=", f.div([x,y])
print "curl(f)=", f.curl([x,y])
print "div(g)=", g.div([x,y,z])
print "curl(g)=", g.curl([x,y,z])
︡548b63a2-14a9-4ddf-9197-2385cbbe1c77︡{"stdout":"f(0,2)= (4, 0)\n"}︡{"stdout":"g(1,2,3)= (2, 3, 6)\n"}︡{"stdout":"div(f)= 5*x\n"}︡{"stdout":"curl(f)= y\n"}︡{"stdout":"div(g)= 2*x*y + y\n"}︡{"stdout":"curl(g)= (-x^2 + z, 0, -x^2 + 2*x*z)\n"}︡{"done":true}︡
︠750db1cb-363d-4d70-90f0-cfce716e2c42si︠

%hide
%html

<H4><B><FONT COLOR="#030856">
Some linear algebra
</FONT></B></H4>

Sage offers powerful tools for working with vectors and 
matrices.  Here is an example showing how to define them, and 
how to do basic arithmetic operations:
︡b5d3e579-58b1-417a-a524-e97fbe1fec92︡{"hide":"input"}︡{"html":"\n<H4><B><FONT COLOR=\"#030856\">\nSome linear algebra\n</FONT></B></H4>\n\nSage offers powerful tools for working with vectors and \nmatrices.  Here is an example showing how to define them, and \nhow to do basic arithmetic operations:"}︡{"done":true}︡
︠c825594b-2388-4998-8712-319d0cd036f1s︠
# Example showing how to work with matrices & vectors
# Define matrix by rows in brackets [] separated by commas
# Define vector by giving its column entries in [] 
A = matrix([ [1, 4, 1], [0, 1, 3], [0, -2, -5] ])
x = vector([0, 1, 2])
y = vector([6, 7, -12])

# Add the two vectors:
print "x+y =", x+y

# Multiply the matrix with vector "x":
print "A*x =", A*x
︡163e7b64-709a-4c24-b547-8ca62237612f︡{"stdout":"x+y = (6, 8, -10)\n"}︡{"stdout":"A*x = (6, 7, -12)\n"}︡{"done":true}︡
︠51f360c9-fcfa-4de7-ac46-a289b2819dd4is︠
%hide
%html
Here is how to solve a matrix-vector system:
︡abba6097-6a95-4fbf-93ca-6c8448bfa17f︡{"hide":"input"}︡{"html":"Here is how to solve a matrix-vector system:"}︡{"done":true}︡
︠35f055d6-724b-44e8-aa4c-f80aee663010s︠
# To solve the matrix-vector system: A*x = y
answer = A\y
print "x =", answer
︡062c17c2-80af-4263-8a0d-5aa27c987e8c︡{"stdout":"x = (0, 1, 2)\n"}︡{"done":true}︡
︠1d9ac87b-9347-4c21-9240-1f32553131cdsi︠
%hide
%html
To find the eigenvalues and eigenvectors of a matrix:
︡1986f215-9a22-4533-ae80-bd4a11d4a3c6︡{"hide":"input"}︡{"html":"To find the eigenvalues and eigenvectors of a matrix:"}︡{"done":true}︡
︠266fa308-d06a-45b2-afad-d99e1d254a04s︠
# To find eigenvalues of matrix
A.eigenvalues()
A.eigenvectors_right()
︡e0420045-4ae3-491c-8243-5faa6a86a564︡{"stdout":"[1, -3.732050807568878?, -0.2679491924311227?]\n"}︡{"stdout":"[(1, [\n(1, 0, 0)\n], 1), (-3.732050807568878?, [(1, -1.953254218877944?, 3.080966067942896?)], 1), (-0.2679491924311227?, [(1, -0.3544380888143643?, 0.1498031628263346?)], 1)]\n"}︡{"done":true}︡
︠4f750212-112c-4c76-a101-6447187c7203is︠
%hide
%html
It is equally easy to transpose, invert, or raise a matrix to any power:
︡28c9c4b0-1791-4458-8664-61f38077da4c︡{"hide":"input"}︡{"html":"It is equally easy to transpose, invert, or raise a matrix to any power:"}︡{"done":true}︡
︠9111e23e-c0ef-4a02-b089-b068a9b3e7f6s︠
# To transpose a matrix:
transpose(A)
# To raise matrix to n_th power (e.g., n=6):
A^6
# To invert a matrix:
B = A^(-1)
print "A^(-1) =", B
︡1b064f79-49e2-4d75-b392-9c7f223fc3af︡{"stdout":"[ 1  0  0]\n[ 4  1 -2]\n[ 1  3 -5]\n"}︡{"stdout":"[    1   510  1200]\n[    0  -989 -2340]\n[    0  1560  3691]\n"}︡{"stdout":"A^(-1) = [ 1 18 11]\n[ 0 -5 -3]\n[ 0  2  1]\n"}︡{"done":true}︡
︠7c93aad6-51f6-4b86-ab79-e7b5c0e43be8si︠

%hide
%html

<H4><B><FONT COLOR="#030856">
Ordinary differential equations
</FONT></B></H4>

ODEs come in many flavors. Some have closed-form analytical solutions, 
while most can only be solved numerically. In addition, sometimes 
we have only single equations, and other times we have systems of 
ODEs.  Sage provides tools for working with all these situations. 
<P><BR>

The following example shows how to find the general solution of 
the 1st order differential equation $x^\prime = 3/x$.  Note that 
the key step is to rewrite the equation with $0$ on one side: 
$x^\prime - 3/x = 0$.
︡beb293a7-2679-4263-86a1-84695b3c0a4a︡{"hide":"input"}︡{"html":"\n<H4><B><FONT COLOR=\"#030856\">\nOrdinary differential equations\n</FONT></B></H4>\n\nODEs come in many flavors. Some have closed-form analytical solutions, \nwhile most can only be solved numerically. In addition, sometimes \nwe have only single equations, and other times we have systems of \nODEs.  Sage provides tools for working with all these situations. \n<P><BR>\n\nThe following example shows how to find the general solution of \nthe 1st order differential equation $x^\\prime = 3/x$.  Note that \nthe key step is to rewrite the equation with $0$ on one side: \n$x^\\prime - 3/x = 0$."}︡{"done":true}︡
︠1456a423-6573-48c7-9f5d-eebb4975bed3s︠
# Example showing how to solve the 1st ord. ODE: x' = 3/x
t = var('t')
x = function('x')(t)
DE = diff(x,t) - 3/x
desolve(DE, [x,t])
︡8a9e3ccd-8dea-4693-9c90-4db1dc55d597︡{"stdout":"1/6*x(t)^2 == _C + t"}︡{"stdout":"\n"}︡{"done":true}︡
︠4076ea92-7cf6-4bef-a172-b6f12232f563si︠
%hide
%html
It is also possible to solve ODEs that contain arbitrary parameters,
as shown next:
︡78338786-3a97-4d91-bdfc-b22b71be9320︡{"hide":"input"}︡{"html":"It is also possible to solve ODEs that contain arbitrary parameters,\nas shown next:"}︡{"done":true}︡
︠6fec5a0a-3751-42de-a32f-31b109608d5cs︠
# Example showing how to solve a 1st ord. ODE with parameters: y' = -ay(1-by)
t = var('t')
y = function('y')(t)
a, b = var('a  b')
DE = diff(y,t) + a*y*(1 - b*y)
desolve(DE, [y,t])
︡a8ca500b-97c2-401f-abf7-33127826ec4f︡{"stdout":"(log(b*y(t) - 1) - log(y(t)))/a == _C + t\n"}︡{"done":true}︡
︠7f9ace83-c00b-4e4e-b27d-7dcb2c3c2029si︠
%hide
%html
Often, we want to plot direction fields for an ODE. Here is 
how to do it:
︡1895da6d-1747-4cbc-9acc-a10bcde77df0︡{"hide":"input"}︡{"html":"Often, we want to plot direction fields for an ODE. Here is \nhow to do it:"}︡{"done":true}︡
︠39b6a4d0-4424-4cde-8147-011e23a71a43s︠
# Example showing how to plot direction fields for 1st ord. ODE: y' = 2y+4-t
t,y = var('t,y')
plot_vector_field((1,2*y+4-t), (t,0,3), (y,-4,0))
︡0b2ce3eb-7779-44d2-aa63-1ebcb79c0971︡{"file":{"filename":"/projects/5af60d30-8ee0-4c83-b7d2-6a0b68f44907/.sage/temp/compute3-us/5460/tmp_GUaQmN.svg","show":true,"text":null,"uuid":"006a2d64-afae-4ed6-b8f1-1f1bd80d0122"},"once":false}︡{"done":true}︡
︠14a8a5f5-09f7-4ebf-b5df-16e81d796866i︠
%hide
%html
Sage can solve systems of linear 1st order ODEs in analytical form.  
Here are two examples -- one shows how to find the general solution, 
the other shows how to find a particular solution of an initial 
value problem:
︡5a4aab23-6885-4fd7-8179-4d698ebe1008︡{"hide":"input"}︡{"html":"Sage can solve systems of linear 1st order ODEs in analytical form.  \nHere are two examples -- one shows how to find the general solution, \nthe other shows how to find a particular solution:"}︡{"done":true}︡
︠d0f226c8-75a7-4c60-814c-6b3fdcf58aae︠
# Example showing how to find general solution of an ODE system.
# System is 1st order, linear: x'=2x+2y; y'=-x+4y

t = var('t')
x = function('x')(t)
y = function('y')(t)
de1 = diff(x,t) - 2*x - 2*y == 0
de2 = diff(y,t) + x - 4*y == 0
desolve_system ([de1, de2], [x, y])
︡5e01a2a9-2aad-47b6-9423-5b586e770be5︡{"stdout":"[x(t) == -((x(0) - 2*y(0))*sin(t) - cos(t)*x(0))*e^(3*t), y(t) == -((x(0) - y(0))*sin(t) - cos(t)*y(0))*e^(3*t)]\n"}︡{"done":true}︡
︠6f44eb5f-b75f-4303-b880-c27ce16c38a9︠
# Example showing how to find particular solution.
# initial value problem is: x'=2x+2y; y'=-x+4y; x(0)=-2, y(0)=1

t = var('t')
x = function('x')(t)
y = function('y')(t)
de1 = diff(x,t) - 2*x - 2*y == 0
de2 = diff(y,t) + x - 4*y == 0
solution = desolve_system ([de1, de2], [x, y], ics=[0, -2, 1])
solution
︡7093b063-0b35-4d4c-9753-6e524139be20︡{"stdout":"[x(t) == -2*(cos(t) - 2*sin(t))*e^(3*t), y(t) == (cos(t) + 3*sin(t))*e^(3*t)]\n"}︡{"done":true}︡
︠5eac6058-f19d-4414-8e1d-743cbe2e4a0esi︠
%hide
%html
It is also possible to solve ODE systems numerically, as shown 
below:
︡439138e3-2e12-423f-9665-81f48f45fdd4︡{"hide":"input"}︡{"html":"It is also possible to solve ODE systems numerically, as shown \nbelow:"}︡{"done":true}︡
︠caa37f60-eaa3-42fa-9dea-9ddcb1aa2da1s︠
# Example showing how to solve the same system numerically
# Systems is: x'=2x+2y; y'=-x+4y; x(0)=-2, y(0)=1

#from sage.calculus.desolvers import desolve_system_rk4
x, y, t = var('x y t')
de1 = 2*x + 2*y
de2 = -x + 4*y
P = desolve_system_rk4 ([de1, de2], [x, y], ics=[0, -2, 1], ivar=t, end_points=[0,2] )
Q = [ [i,j] for i, j, k in P]  # Data for plot of x vs t
xp = line(Q, color='green', legend_label='$x$', legend_color='green')
Q = [ [i,k] for i, j, k in P]  # Data for plot of y vs t
yp = line(Q, axes_labels=['$t$', '$x,y$'], legend_label='$y$', legend_color='blue')
Q = [ [j,k] for i, j, k in P]  # Data for plot of y vs x
show(xp+yp)
line(Q, axes_labels=['$x$', '$y$'])
︡f3d63292-2464-4a47-9ecf-9aa497ff9a69︡{"file":{"filename":"/projects/5af60d30-8ee0-4c83-b7d2-6a0b68f44907/.sage/temp/compute3-us/1598/tmp_cTHfaA.svg","show":true,"text":null,"uuid":"f89f8c22-2127-4b32-af65-715dc52809c1"},"once":false}︡{"file":{"filename":"/projects/5af60d30-8ee0-4c83-b7d2-6a0b68f44907/.sage/temp/compute3-us/1598/tmp_heFGwZ.svg","show":true,"text":null,"uuid":"84541139-7c47-4207-aacc-0efa7eccf7ca"},"once":false}︡{"done":true}︡
︠2320e03a-de77-4466-9eb6-c7051b254e27is︠
%hide
%html

<H4><B><FONT COLOR="#030856">
A taste of abstract algebra/ Groups
</FONT></B></H4>

The group of all permutations of $n$ elements is a 
symmetric group. For example, the symmetric group of the set 
$\{1,2,3,4\}$ can be constructed using the command:
︡78a8a281-559b-4468-bbe1-e1d49481eec5︡{"hide":"input"}︡{"html":"\n<H4><B><FONT COLOR=\"#030856\">\nA taste of abstract algebra/ Groups\n</FONT></B></H4>\n\nThe group of all permutations of $n$ elements is a \nsymmetric group. For example, the symmetric group of the set \n$\\{1,2,3,4\\}$ can be constructed using the command:"}︡{"done":true}︡
︠503b47df-09f5-475f-9e9b-f654b2ffea2fs︠
G = SymmetricGroup([1,2,3,4])
print G
︡932f52f9-47e6-43b1-a440-ed9818766a35︡{"stdout":"Symmetric group of order 4! as a permutation group\n"}︡{"done":true}︡
︠c568cc3a-2b8e-45e1-af27-fb809713d784is︠
%hide
%html
It is now possible to find the number of elements in the 
group, and to list all of them:
︡4aaec734-1c5b-4976-9c2b-632e52edf980︡{"hide":"input"}︡{"html":"It is now possible to find the number of elements in the \ngroup, and to list all of them:"}︡{"done":true}︡
︠62677749-9ab1-4dde-a842-d90f6597f3f2︠
G.cardinality()  # Number of elements in G.
G.list()         # List of elements in G.
︡7b329cbb-184e-4342-a7a5-e02bfb52d819︡{"stdout":"24\n"}︡{"stdout":"[(), (1,2), (1,2,3,4), (1,3)(2,4), (1,3,4), (2,3,4), (1,4,3,2), (1,3,4,2), (1,3,2,4), (1,4,2,3), (1,2,4,3), (2,4,3), (1,4,3), (1,4)(2,3), (1,4,2), (1,3,2), (1,3), (3,4), (2,4), (1,4), (2,3), (1,2)(3,4), (1,2,3), (1,2,4)]\n"}︡{"done":true}︡
︠12529e93-3452-4560-80bf-047a8b43eb52s︠
s = G.random_element(); s  # Pick a random element and print it.
t = G.random_element(); t  # Another random element.
s*t
t*s
︡0a66d06c-d95d-4e55-84e4-d12b93d80df6︡{"stdout":"(1,4,2,3)\n"}︡{"stdout":"(1,4)\n"}︡{"stdout":"(2,3,4)\n"}︡{"stdout":"(1,2,3)\n"}︡{"done":true}︡









