︠215f28e8-c641-4bea-a4a1-4fea37a91f2asi︠
%hide

latex.add_to_preamble('\usepackage{color}')
latex.add_to_preamble('\definecolor{blue}{rgb}{0,0,1}')
latex.add_to_preamble('\definecolor{dkblue}{rgb}{0,0,0.3}')
latex.add_to_preamble('\definecolor{red}{rgb}{1,0,0}')
latex.add_to_preamble('\definecolor{mdred}{rgb}{0.75,0,0}')
latex.add_to_preamble('\definecolor{dkred}{rgb}{0.4,0.01,0.01}')
latex.add_to_preamble('\definecolor{dkgreen}{rgb}{0.01,0.3,0.01}')
latex.add_to_preamble('\setlength{\parindent}{0in}')
︡6003cf1e-bd7a-48f9-a340-4e04da5c98c2︡{"hide":"input"}︡{"done":true}︡
︠d21a4842-4763-4a3d-81f4-bb443e0285f2si︠
%hide
%latex

%\begin{center}
\section*{\color{dkred}Sage for multivariate calculus}
%\end{center}

\noindent
In this module we will work with scalar functions of multiple variables, 
and also with vector-valued functions of one or more variables.
Sage provides extensive support for doing calculus with such functions.  \\[-6pt]

Here are some topics we will cover:
\begin{enumerate}
  \item Defining multivariate scalar functions, and taking their derivatives
  \item Finding their local minimums/maximums
  \item Defining vector-valued functions and taking their derivatives
  \item How to compute gradient, divergence, and curl
  \item Geometric illustration of the meaning of partial derivatives
  \item Multivariable integration
\end{enumerate}

︡9f512750-8284-4cf5-ad46-db1832b90dab︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpy3icCy.png","show":true,"text":null,"uuid":"525f8e56-3207-4366-af66-343d699c77fe"},"once":false}︡{"done":true}︡
︠7ea111da-823d-49c1-8c2f-5a300eb97014si︠
%hide
%latex

{\bf \color{mdred}Functions of 2 or more variables} \\[6pt]
There are several ways to define them.
Here we focus on the easiest and most intuitive ways. \\
The example below defines a function, evaluates it at a point, 
and takes its partial derivatives.  \\

\color{dkgreen}
Example 1: Let $f(x,y,z)=(x^2+y^2)z$. Evaluate the function at 
some point in its domain, and find its 1st partial derivatives.
︡e02fc09d-d648-4fcd-97f9-b130c564063b︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpK1ZVfK.png","show":true,"text":null,"uuid":"de06ddc1-2e78-4931-b85a-423eb37b1d48"},"once":false}︡{"done":true}︡
︠e4b73cad-4f88-4b04-9d69-7e2f8f1a8e94︠
# A function of 3 variables: 
f(x,y,z) = (x^2 + y^2) * z

# Can evaluate the function at any point:
f(1,0,2)

# There are several ways to get partial derivatives:
f.diff(x)
f.derivative(x)
derivative(f,x)
derivative(f,z)
# Note that the answer is returned in function notation, 
# with inputs to the left and outputs to the right of the arrow.

# It is easy to compute the gradient of f:
f.gradient()
︡d4dee503-fa7b-4ace-9b83-a9a3280ad674︡{"stdout":"2\n"}︡{"stdout":"(x, y, z) |--> 2*x*z\n"}︡{"stdout":"(x, y, z) |--> 2*x*z\n"}︡{"stdout":"(x, y, z) |--> 2*x*z\n"}︡{"stdout":"(x, y, z) |--> x^2 + y^2\n"}︡{"stdout":"(x, y, z) |--> (2*x*z, 2*y*z, x^2 + y^2)\n"}︡{"done":true}
︠fea7185e-e994-4e85-8b7e-e1f84fa74859i︠
%hide
%latex
\color{dkblue}
Exercise I: Define each of the following functions, and find 
their 1st partial derivatives.
\begin{enumerate}
  \item $z=\sin(x^2 y - 5x)$.
  \item $f(x_1,x_2,x_3,x_4)=\sin x_1 \cdot \cos x_2 + x_3 \cdot x_4$
  \item Find $g_{xy}$ for $g(x,y) = x^2 y + \cos y$
\end{enumerate}
︡60012320-f6cb-4cb4-bc19-9dba72eafcac︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpQqP_ZG.png","show":true,"uuid":"25b97b3f-7283-4501-be9b-4334bb5957f7"},"once":false}︡{"done":true}
︠d05417ef-c136-411a-b34c-da5b6d8f23dasi︠
%hide
%latex

{\bf \color{mdred}An optimization example in 2 variables} \\[6pt]
Find the local minimum/maximum values of 
$$ g(x,y) = 100(y - x^2)^2 + (1 - x)^2 $$
The solution strategy for 2-variable problems of this type is 
to find the critical points (i.e., where $\nabla g=0$), and 
check the sign of the 2nd derivative $g_{xx}$, and the sign of 
of $g_{xx}\cdot g_{yy} - g_{xy}^2$. This latter quantity is the determinant 
of the Hessian matrix of $g$.
︡1ce39b69-66f0-4faa-8d20-a014ffac6957︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpzAV9A6.png","show":true,"text":null,"uuid":"d0be1ce3-c5ce-425f-a85b-cd84ad655624"},"once":false}︡{"done":true}︡
︠9384191c-df1e-4ff6-8f6c-078c02674ecd︠
# Step 0: Define the function:
g(x,y) = 100*(y - x^2)^2 + (1 - x)^2

# Step 1: Find the points where the gradient of g = 0:
gx(x,y) = derivative(g,x)        # Find derivative of g with respect to x.
gy(x,y) = derivative(g,y)        # Derivative of g with respect to y.
results = solve([gx == 0, gy ==0], (x,y), solution_dict=True)
    # Note that results[0] is a dictionary containing the solutions
xval = results[0][x]
yval = results[0][y]
print "Gradient is 0 at: x =", xval, "y =", yval

# Step 2: Check sign of gxx and determinant of Hessian matrix at those points:
gh = g.hessian()                  # Find the Hessian matrix (in function form)
gh(xval,yval).determinant()       # Compute its determinant at (1,1)
gxx(x,y) = derivative(gx,x)       # Find gxx as function of (x,y)
gxx(xval,yval)                    # Compute its value at (1,1)

# Step 3: Since the Hessian and gxx are both positive at (1,1), it is a local minimum.
# Answer: g(1,1)=0 is a local minimum of the given function g(x,y).
#         Since grad(g) != 0 anywhere else, there is no other 
#         local minimum/maximum.
︡ba82025c-02cf-47da-8e14-02db9c5bd170︡{"stdout":"Gradient is 0 at: x = 1 y = 1\n"}︡{"stdout":"400\n"}︡{"stdout":"802\n"}︡{"done":true}︡
︠b45119c5-4c4e-4eb3-8018-e21d892fd4besi︠
%hide
%latex

{\bf \color{mdred}Vector functions} \\[6pt]
Defining vector-valued functions is also easy, but perhaps less intuitive.
It is important to keep the distinction between independent variable(s), 
and the vector components, clear in your own mind.  For example, the 
simplest vector functions would have 1 variable, but possibly 
several vector components.  Some examples follow: \\
${\bf f}(x) = \left[\begin{array}{c} \cos x  \\ 2x-1  \\ x^2 \end{array}\right ]$, \mbox{~~}  
${\bf g}(x) = \left[\begin{array}{c} e^{-x}  \\ x  \\ \sin x  \\ 3x \sin x \\ -x^2 \end{array}\right ]$ \\
Notice that this ${\bf f}(x)$ can be thought of as a parametric curve 
in 3 dimensions, whereas ${\bf g}(x)$ is a "hyper-curve" in 
5 dimensions, which we cannot visualize or plot. \\

In the Sage segment below, we define each of these functions and 
take their derivatives.  Note that each derivative is also a 
vector function of the same length, whose components Sage 
shows in its output.

︡0934afbd-1cb3-4608-a07f-588e60080e8d︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpCTs3pi.png","show":true,"text":null,"uuid":"86248fe7-ac63-4bd7-bc2b-419efcf6fdb6"},"once":false}︡{"done":true}︡
︠24b7a16a-98f1-49b6-ac28-f1f03f1b6b77︠
# It is important to keep the distinction between independent variable(s), 
# and the vector components, clear in your own mind.

x = var('x')   # The single independent variable

# (1) A 3-component vector:
fv = vector( (cos(x), 2*x-1, x^2) )
fv(x=4)   # Evaluate fv at x=4. Note that fv(4) also currently works, but is to be discontinued soon.
derivative(fv, x)  # 1st derivative of fv (which is also a vector function).
parametric_plot(fv, (x,-2,3), thickness='3')  # Just to see what the function looks like.

# (2) A 5-component vector and its derivative:
gv = vector( (e^-x, x, sin(x), 3*x*sin(x), -x^2) )
derivative(gv, x)
︡384d7a1b-ef90-4617-b4a1-563789332764︡{"stdout":"(cos(4), 7, 16)\n"}︡{"stdout":"(-sin(x), 2, 2*x)\n"}︡{"file":{"filename":"2bca8bf1-9912-4c8f-bdf7-4e2db09d3caf.sage3d","uuid":"2bca8bf1-9912-4c8f-bdf7-4e2db09d3caf"}}︡{"stdout":"(-e^(-x), 1, cos(x), 3*x*cos(x) + 3*sin(x), -2*x)\n"}︡{"done":true}
︠e7c64206-3879-4fd2-b92a-3486a1fd610cis︠
%hide
%latex

{\bf \color{mdred}Vector functions of multiple variables} \\[6pt]
When the independent variable of a vector function is also 
a vector, we have the more general case.  Here are some examples \\
${\bf f}(x,y) = \left[\begin{array}{c}  x^2 + y^2  \\ 3 x y \end{array}\right ]$, \mbox{~~}  
${\bf g}(x,y,z) = \left[\begin{array}{c} x^2 y  \\ x^2 z  \\ y z \end{array}\right ]$ \\

Such functions offer more interesting and fun variations 
for applying familiar calculus concepts!  For example, taking 
their derivatives can be approached in the form of a "divergence" 
or a "curl," with very different conceptual meaning and results. \\

The Sage segment below defines each of these functions, 
evaluates them at a point, and computes their divergence and curl. 
︡52c8dce6-de11-43bd-9fa8-2bf15575229a︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpZ_VEke.png","show":true,"text":null,"uuid":"3f1e960d-fcfa-4736-bca7-2f63399de184"},"once":false}︡{"done":true}︡
︠193caeda-c87b-4828-b1b4-ee09e6bf0b4cs︠
# Here's how to define vector functions of multiple variables. 
# Notice, it is an intuitive generalization of the previous syntax.
#
x, y, z = var('x, y, z')
f = vector( (x^2 + y^2, 3*x*y))
g = vector( (x^2 * y, x^2 * z, y*z) )

# You can evaluate the functions at any specific point by 
# similarly generalizing the single variable syntax.
# (I've used the print command just to show the output clearly):
print "f(0,2)=",  f(x=0, y=2)
print "g(1,2,3)=", g(x=1, y=2, z=3)

# Find divergence and curl. Note, the syntax requires 
# specifying the names of the variables in order.  (Again, 
# the print command is just to show the output clearly):
print "div(f)=", f.div([x,y])
print "curl(f)=", f.curl([x,y])
print "div(g)=", g.div([x,y,z])
print "curl(g)=", g.curl([x,y,z])
︡61f51c53-63ee-441e-89a6-fd28e797f124︡{"stdout":"f(0,2)= (4, 0)\n"}︡{"stdout":"g(1,2,3)= (2, 3, 6)\n"}︡{"stdout":"div(f)= 5*x\n"}︡{"stdout":"curl(f)= y\n"}︡{"stdout":"div(g)= 2*x*y + y\n"}︡{"stdout":"curl(g)= (-x^2 + z, 0, -x^2 + 2*x*z)\n"}︡{"done":true}︡
︠8d32cbe6-d44e-4c85-8061-2402d051a1bfis︠
%hide
%latex

Of course, these vector functions also have simple partial 
derivatives with respect to each component of the independent 
variable, as shown below:
︡f9384df3-398a-45c6-aff3-c161f4ffde48︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpaHKuUc.png","show":true,"text":null,"uuid":"1ba65b35-dc73-4096-8ccc-6cf210d10314"},"once":false}︡{"done":true}︡
︠780c6b43-0c47-40b6-ba1d-f88f8c5bf938s︠
# It is easy to find partial derivatives. Note that each 
# partial derivative is itself a vector function.
print "partial derivatives of f:"
derivative(f, x)  # partial of f with respect to x.
derivative(f, y)  # partial of f with respect to y.

print " "
print "partial derivatives of g:"
derivative(g, x)  # partial of g with respect to x.
derivative(g, y)  # partial of g with respect to y.
derivative(g, z)  # partial of g with respect to z.
︡dab6355c-ac17-4e5f-bfc6-37a2a0084f25︡{"stdout":"partial derivatives of f:\n"}︡{"stdout":"(2*x, 3*y)\n"}︡{"stdout":"(2*y, 3*x)\n"}︡{"stdout":" \n"}︡{"stdout":"partial derivatives of g:\n"}︡{"stdout":"(2*x*y, 2*x*z, 0)\n"}︡{"stdout":"(x^2, 0, z)\n"}︡{"stdout":"(0, x^2, y)\n"}︡{"done":true}︡
︠c0b6eae2-ac78-4e11-8d2c-70e346747816si︠
%hide
%latex

{\bf \color{mdred}Geometric illustration of the meaning of partial derivatives} \\[6pt]
One way to conceptualize the meaning of partial derivatives is 
to look at tangent lines on the surface plot of a 2-variable function.  
Given any point on the surface, the tangent line parallel to the 
$x$-axis represents the $x$-partial derivative, and similarly for 
the $y$-direction.  \\[-6pt]

For example, consider the function: $f(x,y) = \sin(x) + \cos(y)$.  \\[6pt]
The graph below shows the function, together with tangent lines 
parallel to each of the axis directions at a sample point.  The 
sample point can be easily changed by changing one line in the code.

︡2e18dea7-4814-4fb7-808c-95b9328c9347︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpSbBG3F.png","show":true,"text":null,"uuid":"07e1fbbc-a909-455f-a1e6-dbe986b2b6e5"},"once":false}︡{"done":true}︡
︠870dda19-f40b-42f4-bdd2-72cfa7f5ec2b︠
# Example to illustrate the meaning of partial derivatives in a 2-variable function.
# x-derivative gives tangent line parallel to x-direction;
# y-derivative gives tangent line parallel to y-direction.

x, y, t = var('x y t')
fs = sin(x) + cos(y)      # An example function
Q = plot3d(fs, (x,-3,3), (y,-3,3) )
fsx = derivative(fs,x)    # Derivative in x-direction
fsy = derivative(fs,y)    # Derivative in y-direction
xp = 1.5; yp = -1         # Some sample point in the domain of interest

# Tangent in x-direction in parametric form: (xp+t, yp, fsx*t+fs)
R = parametric_plot( (t+xp, yp, t*fsx(x=xp,y=yp)+fs(x=xp,y=yp)), (t,-3,3), thickness=3, color='green' )
# Tangent in y-direction in parametric form: (xp, yp+t, fsy*t+fs)
S = parametric_plot( (xp, t+yp, t*fsy(x=xp,y=yp)+fs(x=xp,y=yp)), (t,-3,3), thickness=3, color='yellow' )
T = point([xp,yp,fs(x=xp,y=yp)], size=10, color='black')
show(Q+R+S+T)

︡1544ae2d-a04d-4d72-98f4-9f25ab8f3e68︡{"file":{"filename":"afc4d7a0-1739-4b2e-97a3-f06efff4214b.sage3d","uuid":"afc4d7a0-1739-4b2e-97a3-f06efff4214b"}}︡{"done":true}︡
︠1032a4b5-7a3b-4059-ae12-ade9f81da906is︠
%hide
%latex

The same example is given below in a completely interactive form.  
The user can make changes by entering information in the input 
boxes.
︡98be24b2-b983-4424-8646-911bab4c15c6︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmp7YV2UW.png","show":true,"text":null,"uuid":"82ceb219-977f-45fd-a9f1-63a3469fe68b"},"once":false}︡{"done":true}︡
︠15e02a3b-bd0a-43c3-95ce-b8349cfa5234si︠
%hide
x, y, t = var('x y t')
#input_grid?
@interact
def _(fs=input_box(sin(x)+cos(y), 'Enter 2-variable function: f(x,y)=', width=60), xr=input_grid(1,2, default=[-3,3], label='Enter xmin, xmax', width=8), yr=input_grid(1,2, default=[-3,3], label='Enter ymin, ymax', width=8), xyp=input_grid(1,2, default=[1.25,0.5], label='Enter (x,y) coordinates of tangency point:', width=8)):
    Q = plot3d(fs, (x,xr[0][0],xr[0][1]), (y,yr[0][0],yr[0][1]) )
    fsx = derivative(fs,x)    # Derivative in x-direction
    fsy = derivative(fs,y)    # Derivative in y-direction
    xp = xyp[0][0]
    yp = xyp[0][1]
    # Tangent in x-direction in parametric form: (xp+t, yp, fsx*t+fs)
    R = parametric_plot( (t+xp, yp, t*fsx(x=xp,y=yp)+fs(x=xp,y=yp)), (t,-3,3), thickness=3, color='green' )
    # Tangent in y-direction in parametric form: (xp, yp+t, fsy*t+fs)
    S = parametric_plot( (xp, t+yp, t*fsy(x=xp,y=yp)+fs(x=xp,y=yp)), (t,-3,3), thickness=3, color='yellow' )
    T = point([xp,yp,fs(x=xp,y=yp)], size=10, color='black')
    show(Q+R+S+T)
︡a53b3d57-f522-41ca-880a-a4729d32c35b︡{"hide":"input"}︡{"interact":{"controls":[{"control_type":"input-box","default":"cos(y) + sin(x)","label":"Enter 2-variable function: f(x,y)=","nrows":1,"readonly":false,"submit_button":null,"type":null,"var":"fs","width":60},{"control_type":"input-grid","default":[["-3","3"]],"label":"Enter xmin, xmax","ncols":2,"nrows":1,"var":"xr","width":8},{"control_type":"input-grid","default":[["-3","3"]],"label":"Enter ymin, ymax","ncols":2,"nrows":1,"var":"yr","width":8},{"control_type":"input-grid","default":[["1.25000000000000","0.500000000000000"]],"label":"Enter (x,y) coordinates of tangency point:","ncols":2,"nrows":1,"var":"xyp","width":8}],"flicker":false,"id":"d770ca56-0ad8-4876-b120-352a16147ed7","layout":[[["fs",12,null]],[["xr",12,null]],[["yr",12,null]],[["xyp",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠c3e24af9-a56b-4037-8551-2a88ee0c964fis︠
%hide
%latex

The collection of all tangent lines through the same point 
comprises the tangent plane.  Let $f(x,y)$ be the 
function and $(c,d)$ the point of tangency.  Then the 
parametric form of the tangent plane is: \\[6pt]
$x = s+c, ~~ y = t+d, ~~ z = s\cdot f_x(c,d) + t\cdot f_y(c,d) + f(c,d)$ \\[6pt]
for $s, t \in (-\infty,\infty)$.  \\[6pt]
The code segment below shows the function, together with 
its tangent plane at the sample point.

︡28b9e294-4d63-40da-893b-535c0a4e388c︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpS8uuTz.png","show":true,"text":null,"uuid":"fc8de173-159c-4385-a642-96cd39bf7706"},"once":false}︡{"done":true}︡
︠3c393356-cea5-42e1-9546-123ac3dcea61s︠
# Tangent plane example.  Here the tangent plane at the point (c,d) has 
# the parametric form: x = s+c, y = t+d, z = s*fx(c,d) + t*fy(c,d) + f(c,d)
# for $s, t \in (-\infty,\infty)$.

x, y, s, t = var('x y s t')
fs = sin(x) + cos(y)
Q = plot3d(fs, (x,-3,3), (y,-3,3) )
fsx = derivative(fs,x)    # Derivative in x-direction
fsy = derivative(fs,y)    # Derivative in y-direction
xp = 2; yp = -0.5         # Some sample point in the domain of interest
R = parametric_plot( (s+xp, t+yp, s*fsx(x=xp,y=yp)+t*fsy(x=xp,y=yp)+fs(x=xp,y=yp)), (t,-3,3), (s,-3,3), thickness=3, color='yellow', opacity=0.3 )
T = point([xp,yp,fs(x=xp,y=yp)], size=10, color='black')
show(Q+R+T)
︡5fbe7143-cf51-472d-84ee-3f8ab89cd297︡{"file":{"filename":"063b5208-8ea9-460a-b91f-1959038a71c6.sage3d","uuid":"063b5208-8ea9-460a-b91f-1959038a71c6"}}︡{"done":true}︡
︠fc57c4ab-74b3-4e18-88b3-0a9c5fa8a5d6si︠

%hide
%latex

{\bf \color{mdred}Multivariable integration} \\[6pt]
Recall, for single-variable functions, Sage offers 
both definite and indefinite integration capability.  For example, 
if $f(x)=x^2$, then we can find $\displaystyle \int f(x) dx$ and 
$\displaystyle \int_0^1 f(x) dx$ as follows
︡622ee1d5-6ab9-4805-8ed2-23d3c341a078︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpETQOQS.png","show":true,"text":null,"uuid":"0ee30540-9dbf-4a06-8311-3409248986c2"},"once":false}︡{"done":true}︡
︠ed2f22f1-2781-4324-a38a-d25e58d00e74s︠
x = var('x')
integral(x^2, x)          # Indefinite integral
integral(x^2, (x,0,1))    # Definite integral
︡27a0cb68-6e63-412a-a389-4bfda1cfcc35︡{"stdout":"1/3*x^3"}︡{"stdout":"\n"}︡{"stdout":"1/3\n"}︡{"done":true}︡
︠dc9bec02-b3e2-43bf-9a38-6f4938ffe42esi︠
%hide
%latex
A simple, intuitive extension of this syntax provides integration 
in multiple variables.  For example, let $f(x,y)=x^2 \sin(y)$.  
Then $\displaystyle \int \int f(x,y) \; dy \; dx$ and 
$\displaystyle \int_0^1 \int_{0}^{\pi} f(x,y) \; dy \; dx$ 
can be found via
︡0e5af53d-bf24-44a1-8b52-4b9292b7af65︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmp5siDZC.png","show":true,"text":null,"uuid":"5afacf08-90ea-421d-b565-b848022ebb9d"},"once":false}︡{"done":true}︡
︠379b9624-3408-4357-918c-9d466adfc0a1s︠
x, y = var('x, y')
integral(integral(x^2*sin(y), y), x)               # Indefinite integral
integral(integral(x^2*sin(y), (y,0,pi)), (x,0,1))  # Definite integral
︡53baf7e0-af85-4235-8916-e3bef275b1a8︡{"stdout":"-1/3*x^3*cos(y)\n"}︡{"stdout":"2/3\n"}︡{"done":true}︡
︠01a96eb2-66ed-49ce-a615-060c831adca3is︠
%hide
%latex
{\color{blue}Example 2}: 
Although not the easiest method, it is possible to find 
a formula for the area of a circle using double integration.  
If the radius is $r$, then the area is: 
$\displaystyle \int_{-r}^r \int_{-\sqrt{r^2-x^2}}^{\sqrt{r^2-x^2}} \; dy \; dx$.  
The Sage implementation is shown below
︡d15abbac-3d96-4388-9068-6641aaf02f10︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpuZahse.png","show":true,"text":null,"uuid":"678d683c-1c55-4500-b483-989815a62a56"},"once":false}︡{"done":true}︡
︠143a6bb6-7ec7-4d72-9290-b73e2af5325es︠
# Formula for the area of a circle using double integration
x,y, r = var('x y r')
assume(r>0)               # This is needed to to prevent complaints from Sage!
integral(integral(1, (y,-sqrt(r^2-x^2),sqrt(r^2-x^2))), (x,-r,r))
︡ee444220-95c2-4074-a8a2-9b4b09e58fbc︡{"stdout":"pi*r^2\n"}︡{"done":true}









